define(['bridge', 'translate', 'jquery', 'jqueryui', 'hashcode'], function (Bridge, Translate, $, _, hashcode) {

  /**
   * @class DialogButtonEditor
   * @singleton
   */
  return new function DialogButtonEditor () {

    /** @private */
    var element, form, entry, onDialogSubmit;

    /**
     * @constructor
     */
    this.initialize = function () {
      element = $("#dialog-edit-custom-button");
      form = {
        name: $("#edit-custom-button-name"),
        category: $("#edit-custom-button-category"),
        description: $("#edit-custom-button-description"),
        icon: $("#edit-custom-button-icon"),
        icon_select: $("#edit-custom-button-icon-select"),
        icon_image: $("#edit-custom-button-image"),
        code: $("#edit-custom-button-code"),
        cancel: $("#button-edit-custom-cancel"),
        save: $("#button-edit-custom-save")
      };
      // The HTMLInputElement[type=file] does not give access to the absolute file
      // path. Because of this, we simulate it with a text input and a button using 
      // the SketchUp API method.
      form.icon_select.on("click", function() {
        Bridge.get("openpanel", form.icon.val()).then(function(filename){
          if (!filename) { return; }
          form.icon.val(filename);
          form.icon_image.attr("src", filename);
        });
      });
      form.icon.on("click", function() {
        form.icon_select.trigger("click");
      });
      // Cancel button
      form.cancel.on("click", function() {
        element.hide();
      });
      // Save button
      form.save.on("click", function() {
        if (entry) {
          entry.name = form.name.val();
          if (!entry.id) { entry.id = hashcode(entry.name + entry.description + entry.icon); }
          entry.category = form.category.val();
          entry.description = form.description.val();
          entry.icon = form.icon.val();
          entry.code = form.code.val();
          // Save entry to SketchUp
          Bridge.get("save_custom_button", entry.id, entry.code, 
            {name: entry.name, category: entry.category, description: entry.description, icon: entry.icon}
          ).then(
            function(success) {
              if (success) {
                if (typeof onDialogSubmit === "function") {
                  onDialogSubmit(entry);
                }
                element.hide();
              } else {
                // Show warning about error
                form.code.css({border: "3px red solid"});
                setTimeout(function(){ form.code.css({border: ""}); }, 500);
              }
          });
        }
      });
      element.hide();
    };


    /**
     * Shows the dialog for editing a custom button.
     * @param {function(object)} callback - Called after editing an entry.
     * @param {object} old_entry
     */
    this.show = function(callback, old_entry) {
      onDialogSubmit = callback;
      // Create a new entry if no existing entry given.
      if (!old_entry) { entry = {}; }
      else { entry = old_entry; }
      // Fill in the existing data.
      form.name.val(entry.name); //.attr("disabled", !!entry.id);
      form.category.val(entry.category);
      form.description.val(entry.description);
      form.icon.val(entry.icon);
      form.icon_image.attr("src", entry.icon);
      form.code.val(entry.code);
      // Show the dialog.
      element.show();
    };


  };


});
