// Main function
requirejs(['./bridge', 'jquery', 'jqueryui', './translate', 'toolbareditor'], function (Bridge, $, _, Translate, ToolbarEditor) {

    // 4. Fallback for svg-resources to png images.
    if (navigator.userAgent.match(/MSIE|Trident/i)) {
        // Internet Explorer (even if it can render svg, it does not scale them properly).
        $('img').each(function (index, img) {
            img.src = img.src.replace(/\.svg$/, '.png');
        });
    } else {
        // Fallback on load failure.
        $('img').on('error', function () {
            var img = $(this),
                src = img.attr('src');
            img.attr('src', src.replace(/\.svg$/, '.png'));
        });
    }

    // 5. Catch uncaught errors in the WebDialog and send them to the console.
    // There are some errors by the ACE editor (when double-clicking or selection
    // that goes outside the editor) that would be silent in a normal Internet Explorer
    // but cause popups in SketchUp.
    window.onerror = function(messageOrEvent, source, lineNumber, columnNumber, errorObject) {
        window.console.log([messageOrEvent, source, lineNumber, columnNumber, errorObject]);
        if (!errorObject) {
            errorObject = new Error();
            errorObject.name = 'Error';
            errorObject.message = messageOrEvent;
            errorObject.fileName = source;
            errorObject.lineNumber = lineNumber;
            errorObject.columnNumber = columnNumber;
        }
        Bridge.error(errorObject);
        return true;
    };

    // 6. Load content

    // Load translations and translate the html document.
    Bridge.call('translate');

    Promise.all([
        Bridge.get('get_custom_buttons').then(function (buttons) {
            ToolbarEditor.loadCustomButtons(buttons);
        }),
        Bridge.get('get_available_buttons').then(function (buttons) {
            ToolbarEditor.loadButtons(buttons);
        })
    ]).then(function () {
        Bridge.get('get_toolbars').then(function (toolbars) {
            ToolbarEditor.loadToolbars(toolbars)
        })
    });

});
