define(['bridge', 'translate', 'jquery', 'jqueryui', 'toolbarbutton'], function (Bridge, Translate, $, _, ToolbarButton) {


  /**
   * @class Toolbar
   * @param {string} name
   * @param {numeric} id
   */
  return function Toolbar (id, name) {

    /** @private */
    var toolbar = this;
    var header, title, titleEdit, buttonRemove, buttonEdit, buttonWrapper, content, statusEditName = false;

    /** @public */
    toolbar.id = id;
    toolbar.name = (typeof name == "string") ? name :
      Translate.get("Toolbar") + " " + toolbar.id;
    toolbar.buttons = [];

    /**
     * Initializes the new toolbar and adds it to the UI.
     * @constructor
     * @private
     */
    function initialize () {
      // Header with buttons.
      header = $("<h3/>").
        attr({"class": "toolbar header"});
      header.get(0).toolbar = toolbar; // reference
      title = $("<span/>").text(toolbar.name);
      titleEdit = $("<input/>").
        attr({"type": "text", "value": toolbar.name}).
        hide().
        on("keydown", function(event) {
          // Enter
          if (event.which == 13) {
            toolbar.finishEditName();
            event.preventDefault();
          }
          event.stopPropagation();
        }).
        on("blur", function() {
          toolbar.finishEditName();
          statusEditName = false;
        });
      // Button to remove the toolbar.
      buttonRemove = $("<button/>").
        attr({"class": "symbolic-button", "title": Translate.get("Remove this toolbar")}).
        append($("<img/>").attr("src", "../images/icon_close.png")).
        on("click", function() {
          toolbar.remove();
        });
      // Button to edit a toolbar's name.
      buttonEdit = $("<button/>").
        attr({"class": "symbolic-button", "title": Translate.get("Edit the toolbar name")}).
        append($("<img/>").attr("src", "../images/icon_edit.png")).
        on("click", function() {
          !statusEditName ? toolbar.editName() : toolbar.finishEditName();
          statusEditName = !statusEditName;
        });
      buttonWrapper = $("<span/>").
        attr({"class": "right"}).
        append(buttonEdit, buttonRemove);
      header.append(title, titleEdit, buttonWrapper, $("<div/>").attr({"class": "clear"}) );

      //Content
      content = $("<ul/>").
        attr({"class": "toolbar content"});
      content.get(0).toolbar = toolbar; // reference

      // Add to toolbars list.
      $("#custom-toolbars").append(header, content);

      // Only after the content has been added to the DOM, we can make it sortable.
      content.sortable({
        connectWith: ".toolbar",
        revert: true,
        // Refresh the toolbar buttons array:
        change: function(event, ui) { update() },
        stop: function(event, ui) { update() }, // Also triggered when items on droppable.
        scroll: false,
        appendTo: document.body
        // Check that the dragged element is allowed to be dropped.
        /*beforeStop: function(event, ui) {
          // Check that it is no duplicate (except separator). When SketchUp loads toolbars, duplicates show a wrong icon.
          var id = Number(ui.item.attr("name"));
          if ( $.inArray(id, toolbar.buttons) >= 0 && id !== 0) { ui.item.remove(); }
        },*/
      });
    };

    /**
     * Refreshes the list of buttons of this toolbar.
     * @private
     */
    function update () {
      toolbar.buttons = content.find(".toolbar-button").map(function() {
        return this.getAttribute("name")
      }).get();
    };

    /**
     * Select the panel of this toolbar.
     * @public
     */
    toolbar.activate = function activate () {
      $("#custom-toolbars").parent().scrollTop(header.offset().top);
    };

    /**
     * Edit mode of this toolbar's name.
     */
    toolbar.editName = function editName () {
      titleEdit.show().focus();
      title.hide();
      /* Set caret to end */
      var e = titleEdit.get(0);
      var pos = e.value.length;
      /* DOM 3.0 */
      if('selectionStart' in e){
        e.selectionStart = pos;
        e.selectionEnd = pos;
      }
      /* Internet Explorer */
      else if (document.selection) {
        var r = e.createTextRange();
        r.moveEnd('textedit', -1);
        r.moveStart('character', pos);
        r.moveEnd('character', 0);
        r.select();
      }
    };

    /**
     * Finish edit mode of this toolbar's name.
     */
    toolbar.finishEditName = function finishEditName () {
      if (statusEditName) {
        toolbar.name = titleEdit.hide().val();
        title.text(toolbar.name).show();
      }
    };

    /**
     * Add a new button (programmatically, not drag&drop)
     * @param {number} id
     * @public
     */
    toolbar.addButton = function addButton (id, entry) {
      var button = new ToolbarButton(id, entry);
      content.append(button.jQueryElement);
      toolbar.buttons.push(button.id);
    };

    /**
     * Remove this toolbar completely.
     * @public
     */
    toolbar.remove = function remove () {
      header.remove();
      content.remove();
      $(toolbar).trigger("delete");
      Bridge.call("delete_toolbar", toolbar.id)
      delete toolbar;
    };

    initialize();
  };

});
