module AE


  module ToolbarEditor


    def self.get_resource_path(basename)
      if Sketchup.version.to_i >= 16
        if RUBY_PLATFORM =~ /darwin/
          return File.join(PATH, 'images', "#{basename}.pdf")
        else
          return File.join(PATH, 'images', "#{basename}.svg")
        end
      else
        return File.join(PATH, 'images', "#{basename}.png")
      end
    end
    private_class_method :get_resource_path


    def self.create_command()
      # Command
      command = UI::Command.new(TRANSLATE['Toolbar Editor']){
        begin
          AE::ToolbarEditor.show_dialog
        rescue Exception => error
          if defined?(AE::ConsolePlugin)
            AE::ConsolePlugin.error(error)
          else
            $stderr.write(error.message + $/)
            $stderr.write(error.backtrace.join($/) + $/)
          end
        end
      }
      # Icons
      command.large_icon = command.small_icon = get_resource_path('icon')
      # Metadata
      command.tooltip = TRANSLATE["A drag&drop editor to create custom toolbars."]
      return command
    end
    private_class_method :create_command


    def self.register_menu(command)
      UI.menu('Window').add_item(command)
    end
    private_class_method :register_menu


    def self.initialize_ui
      command = create_command()
      register_menu(command)
    end
    private_class_method :initialize_ui


  end


end
